//
// Copyright (c) 2014, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Xml;

using MindFusion.Scheduling;
using ImageAlignment = MindFusion.Drawing.ImageAlignment;


namespace MindFusion.Scheduling.WinForms.Samples.CS.AdvancedItemDrawing
{
	/// <summary>
	/// Represents an ordinary appointment with a round shape.
	/// </summary>
	[ItemViewer(typeof(RoundAppointmentViewer))]
	public class RoundAppointment : Appointment
	{
		/// <summary>
		/// Initializes a new instance of the RoundAppointment class.
		/// </summary>
		public RoundAppointment()
		{
			radius = 10;
		}

		/// <summary>
		/// Creates an exact shallow clone of this appointment.
		/// </summary>
		public override object Clone()
		{
			RoundAppointment clone = new RoundAppointment();

			clone.AllDayEvent = this.AllDayEvent;
			clone.AllowChangeEnd = this.AllowChangeEnd;
			clone.AllowChangeStart = this.AllowChangeStart;
			clone.AllowMove = this.AllowMove;
			clone.DescriptionText = this.DescriptionText;
			clone.EndTime = this.EndTime;
			clone.HeaderText = this.HeaderText;
			clone.Location = this.Location;
			clone.Locked = this.Locked;
#if !PocketPC
			clone.PointedSelectedStyle = this.PointedSelectedStyle.Clone() as Style;
			clone.PointedStyle = this.PointedStyle.Clone() as Style;
#endif
			clone.Priority = this.Priority;
			clone.Reminder = this.Reminder;
			clone.SelectedStyle = this.SelectedStyle.Clone() as Style;
			clone.StartTime = this.StartTime;
			clone.Style = this.Style.Clone() as Style;
			clone.Tag = this.Tag;
			clone.Task = this.Task;
			clone.Visible = this.Visible;

			foreach (Resource resource in this.Resources)
				clone.Resources.Add(resource);

			foreach (Contact contact in this.Contacts)
				clone.Contacts.Add(contact);

			clone.Radius = this.Radius;

			return clone;
		}

		/// <summary>
		/// Appointment.CopyOccurrence override.
		/// </summary>
		protected override void CopyOccurrence(Item master)
		{
			this.radius = (master as RoundAppointment).radius;
			base.CopyOccurrence(master);
		}

		/// <summary>
		/// Appointment.SaveTo override.
		/// </summary>
		public override void SaveTo(BinaryWriter writer, BinarySerializationContext context)
		{
			base.SaveTo(writer, context);
			context.WriteInt(radius, writer);
		}

		/// <summary>
		/// Appointment.LoadFrom override.
		/// </summary>
		public override void LoadFrom(BinaryReader reader, BinarySerializationContext context)
		{
			base.LoadFrom(reader, context);
			radius = context.ReadInt(reader);
		}

		/// <summary>
		/// Appointment.SaveTo override.
		/// </summary>
		public override void SaveTo(XmlElement element, XmlSerializationContext context)
		{
			base.SaveTo(element, context);
			context.WriteInt(radius, "radius", element);
		}

		/// <summary>
		/// Appointment.LoadFrom override.
		/// </summary>
		public override void LoadFrom(XmlElement element, XmlSerializationContext context)
		{
			base.LoadFrom(element, context);
			radius = context.ReadInt("radius", element);
		}


		/// <summary>
		/// Gets or sets the round rect radius of this appointment.
		/// </summary>
		public int Radius
		{
			get { return radius; }
			set
			{
				if (radius == value)
					return;

				radius = value;

				// Invalidate item visuals
				OnVisualsChanged();

				// Changing the selected style causes
				// recurrent appointments to become exceptions
				if (Recurrence != null)
					Recurrence.MarkException(this, false);
			}
		}


		private int radius;
	}
}
