//
// Copyright (c) 2014, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;

using MindFusion.Scheduling;
using ImageAlignment = MindFusion.Drawing.ImageAlignment;


namespace MindFusion.Scheduling.WinForms.Samples.CS.AdvancedItemDrawing
{
	/// <summary>
	/// An appointment viewer with a round rectangle shape.
	/// </summary>
	public class RoundAppointmentViewer : IItemViewer
	{
		/// <summary>
		/// Initializes a new instance of the RoundAppointmentViewer class.
		/// </summary>
		public RoundAppointmentViewer()
		{
		}

		/// <summary>
		/// IItemViewer.Draw override.
		/// </summary>
		public void Draw(ItemDrawContext context)
		{
			var graphics = context.Graphics;
			var appointment = context.Item as RoundAppointment;
			int radius = appointment.Radius;

			// Calculate visible corners
			bool leftTop = true;
			bool rightTop = true;
			bool leftBottom = true;
			bool rightBottom = true;
			if (context.IsHorizontal)
			{
				if (!context.StartsHere)
				{
					leftTop = false;
					leftBottom = false;
				}

				if (!context.EndsHere)
				{
					rightTop = false;
					rightBottom = false;
				}
			}
			else
			{
				if (!context.StartsHere)
				{
					leftTop = false;
					rightTop = false;
				}

				if (!context.EndsHere)
				{
					leftBottom = false;
					rightBottom = false;
				}
			}

			// Get some frequently used properties
			Rectangle bounds = context.Bounds;
			bounds.Width -= 1;
			bounds.Height -= 1;

			// Save graphics state
			var state = graphics.Save();

			// Enable smoothing
			graphics.SmoothingMode = SmoothingMode.AntiAlias;
			graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;

			// Create a round rectangle GraphicsPath for the item
			var path = CreateRoundRect(
				bounds.X, bounds.Y, bounds.Width, bounds.Height, radius,
				leftTop, rightTop, leftBottom, rightBottom);

			// Set the path for clipping
			var oldClip = graphics.Clip;
			var newClip = new Region(path);
			if (oldClip != null)
				newClip.Intersect(oldClip);
			graphics.Clip = newClip;

			// Fill header
			int headerSize = context.Calendar.ItemSettings.HeaderSize;
			Rectangle r;
			if (headerSize > 0)
			{
				Rectangle headerBounds = bounds;
				headerBounds.Height = headerSize;

				// Set new clip
				var oldClip2 = graphics.Clip;
				var newClip2 = new Region(headerBounds);
				if (oldClip2 != null)
					newClip2.Intersect(oldClip2);
				graphics.Clip = newClip2;

				var headerBrush = context.Style.HeaderBrush.CreateGdiBrush(headerBounds);
				graphics.FillRectangle(headerBrush, headerBounds);
				headerBrush.Dispose();

				// Draw the 'doesn't start here' icon
				r = headerBounds;
				if (!context.StartsHere)
				{
					Point[] poly = new Point[]
					{
						new Point(r.Left + 4, r.Y + r.Height / 2),
						new	Point(r.Left + 8, r.Y + r.Height / 2 - 4),
						new	Point(r.Left + 8, r.Y + r.Height / 2 + 4)
					};

					graphics.FillPolygon(Brushes.Black, poly);
					headerBounds.X += 6;
					headerBounds.Width -= 6;
				}

				// Draw the 'doesn't end here' icon
				r = headerBounds;
				if (!context.EndsHere)
				{
					Point[] poly = new Point[]
					{
						new Point(r.Right - 4, r.Y + r.Height / 2),
						new	Point(r.Right - 8, r.Y + r.Height / 2 - 4),
						new	Point(r.Right - 8, r.Y + r.Height / 2 + 4)
					};

					graphics.FillPolygon(Brushes.Black, poly);
					headerBounds.Width -= 8;
				}

				// Draw the item icons in its header
				int iconSize = 16;
				if (context.Item.RecurrenceState == RecurrenceState.Exception ||
					context.Item.RecurrenceState == RecurrenceState.Occurrence)
				{
					r = headerBounds;
					r.Inflate(-2, -2);
					r.Width = iconSize;
					r.Y = r.Y + r.Height / 2 - iconSize / 2;
					r.Height = iconSize;

					context.DrawRecurrenceIcon(ref r, false);

					headerBounds.X += iconSize + 2;
					headerBounds.Width -= iconSize + 2;
				}

				if (context.Item.Reminder != null)
				{
					r = headerBounds;
					r.Inflate(-2, -2);
					r.Width = iconSize;
					r.Y = r.Y + r.Height / 2 - iconSize / 2;
					r.Height = iconSize;

					context.DrawReminderIcon(ref r, false);

					headerBounds.X += iconSize + 2;
					headerBounds.Width -= iconSize + 2;
				}

				// Draw header text
				r = headerBounds;
				context.DrawHeaderText(ref r, false, false);

				// Restore clip
				graphics.Clip = oldClip2;
				newClip2.Dispose();
			}

			// Fill conents
			Rectangle contentsBounds = bounds;
			contentsBounds.Y += headerSize;
			contentsBounds.Height -= headerSize;

			if (contentsBounds.Width > 0 && contentsBounds.Height > 0)
			{
				var contentsBrush = context.Style.Brush.CreateGdiBrush(
					new RectangleF(contentsBounds.X, contentsBounds.Y, contentsBounds.Width, contentsBounds.Height));
				graphics.FillRectangle(contentsBrush, contentsBounds);
				contentsBrush.Dispose();

				// Get available space for contents
				r = contentsBounds;
				r.X += context.Style.TextLeftMargin;
				r.Width -= (context.Style.TextLeftMargin + context.Style.TextRightMargin);
				r.Y += context.Style.TextTopMargin;
				r.Height -= (context.Style.TextTopMargin + context.Style.TextBottomMargin);

				// Set new clip
				var oldClip2 = graphics.Clip;
				var newClip2 = new Region(r);
				if (oldClip2 != null)
					newClip2.Intersect(oldClip2);
				graphics.Clip = newClip2;

				// Restore clip
				graphics.Clip = oldClip2;
				newClip2.Dispose();

				// Draw contents text
				if (!string.IsNullOrEmpty(appointment.DescriptionText))
					context.DrawFormattedContentsText(ref r, true);

				var pen = new Pen(context.Style.LineColor, 0);
				graphics.DrawLine(pen, contentsBounds.Left, contentsBounds.Top,
					contentsBounds.Right, contentsBounds.Top);
				pen.Dispose();
			}

			// Restore clipping
			graphics.Clip = oldClip;
			newClip.Dispose();

			var leftPen = new Pen(context.Style.BorderLeftColor, 0);
			var left = new GraphicsPath();
			AddLeft(bounds.X, bounds.Y, bounds.Width, bounds.Height, radius,
				leftTop, rightTop, leftBottom, rightBottom, left);
			graphics.DrawPath(leftPen, left);
			left.Dispose();
			leftPen.Dispose();

			var topPen = new Pen(context.Style.BorderTopColor, 0);
			var top = new GraphicsPath();
			AddTop(bounds.X, bounds.Y, bounds.Width, bounds.Height, radius,
				leftTop, rightTop, leftBottom, rightBottom, top);
			graphics.DrawPath(topPen, top);
			top.Dispose();
			topPen.Dispose();

			var rightPen = new Pen(context.Style.BorderRightColor, 0);
			var right = new GraphicsPath();
			AddRight(bounds.X, bounds.Y, bounds.Width, bounds.Height, radius,
				leftTop, rightTop, leftBottom, rightBottom, right);
			graphics.DrawPath(rightPen, right);
			right.Dispose();
			rightPen.Dispose();

			var bottomPen = new Pen(context.Style.BorderBottomColor, 0);
			var bottom = new GraphicsPath();
			AddBottom(bounds.X, bounds.Y, bounds.Width, bounds.Height, radius,
				leftTop, rightTop, leftBottom, rightBottom, bottom);
			graphics.DrawPath(bottomPen, bottom);
			bottom.Dispose();
			bottomPen.Dispose();

			// Dispose the path
			path.Dispose();

			// Restore graphics state
			graphics.Restore(state);
		}

		private void DrawText(string text, RectangleF rect, MindFusion.Text.DrawTextHint hint)
		{
			hint.Graphics.DrawString(text, hint.Font, hint.Brush, rect.Location);
		}

		private static void AddTop(float x, float y, float width, float height, float r,
			bool leftTop, bool rightTop, bool leftBottom, bool rightBottom, GraphicsPath gp)
		{
			float diameter = Math.Min(2 * r, Math.Min(width, height));
			r = diameter / 2;

			if (diameter < width)
				gp.AddLine(x + r, y, x + width - r, y);

			if (r > 0)
			{
				if (rightTop)
				{
					gp.AddArc(x + width - diameter, y, diameter, diameter, 270, 90);
				}
				else
				{
					gp.AddLine(x + r, y, x + width, y);
					gp.AddLine(x + width, y, x + width, y + r);
				}
			}
		}

		private static void AddRight(float x, float y, float width, float height, float r,
			bool leftTop, bool rightTop, bool leftBottom, bool rightBottom, GraphicsPath gp)
		{
			float diameter = Math.Min(2 * r, Math.Min(width, height));
			r = diameter / 2;

			if (diameter < height)
				gp.AddLine(x + width, y + r, x + width, y + height - r);

			if (r > 0)
			{
				if (rightBottom)
				{
					gp.AddArc(x + width - diameter, y + height - diameter, diameter, diameter, 0, 90);
				}
				else
				{
					gp.AddLine(x + width, y + height - r, x + width, y + height);
					gp.AddLine(x + width, y + height, x + width - r, y + height);
				}
			}
		}

		private static void AddBottom(float x, float y, float width, float height, float r,
			bool leftTop, bool rightTop, bool leftBottom, bool rightBottom, GraphicsPath gp)
		{
			float diameter = Math.Min(2 * r, Math.Min(width, height));
			r = diameter / 2;

			if (diameter < width)
				gp.AddLine(x + width - r, y + height, x + r, y + height);

			if (r > 0)
			{
				if (leftBottom)
				{
					gp.AddArc(x, y + height - diameter, diameter, diameter, 90, 90);
				}
				else
				{
					gp.AddLine(x + r, y + height, x, y + height);
					gp.AddLine(x, y + height, x, y + height - r);
				}
			}
		}

		private static void AddLeft(float x, float y, float width, float height, float r,
			bool leftTop, bool rightTop, bool leftBottom, bool rightBottom, GraphicsPath gp)
		{
			float diameter = Math.Min(2 * r, Math.Min(width, height));
			r = diameter / 2;

			if (diameter < height)
				gp.AddLine(x, y + height - r, x, y + r);

			if (r > 0)
			{
				if (leftTop)
				{
					gp.AddArc(x, y, diameter, diameter, 180, 90);
				}
				else
				{
					gp.AddLine(x, y + r, x, y);
					gp.AddLine(x, y, x + r, y);
				}
			}
		}

		/// <summary>
		/// Creates a graphics path of a rounded rectangle with
		/// the specified dimensions and round radius.
		/// </summary>
		private static GraphicsPath CreateRoundRect(
			float x, float y, float width, float height, float r,
			bool leftTop, bool rightTop, bool leftBottom, bool rightBottom)
		{
			GraphicsPath gp = new GraphicsPath();

			try
			{
				AddTop(x, y, width, height, r, leftTop, rightTop, leftBottom, rightBottom, gp);
				AddRight(x, y, width, height, r, leftTop, rightTop, leftBottom, rightBottom, gp);
				AddBottom(x, y, width, height, r, leftTop, rightTop, leftBottom, rightBottom, gp);
				AddLeft(x, y, width, height, r, leftTop, rightTop, leftBottom, rightBottom, gp);

				gp.CloseFigure();
			}
			catch (Exception)
			{
				gp.AddRectangle(new RectangleF(x, y, width, height));
			}
			return gp;
		}
	}
}
